@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -33,13 +33,13 @@
 """
 
 from tvb.adapters.datatypes.db.graph import ConnectivityMeasureIndex
-from tvb.basic.neotraits.api import Attr
 from tvb.basic.logger.builder import get_logger
+from tvb.basic.neotraits.api import Attr
 from tvb.core.adapters.abcuploader import ABCUploader, ABCUploaderForm
 from tvb.core.adapters.exceptions import ParseException, LaunchException
 from tvb.core.entities.storage import transactional
-from tvb.core.neotraits.forms import TraitUploadField, StrField, TraitDataTypeSelectField
 from tvb.core.neocom import h5
+from tvb.core.neotraits.forms import TraitUploadField, StrField, TraitDataTypeSelectField
 from tvb.core.neotraits.uploader_view_model import UploaderViewModel
 from tvb.core.neotraits.view_model import Str, DataTypeGidAttr
 from tvb.datatypes.connectivity import Connectivity
@@ -67,13 +67,12 @@ class ConnectivityMeasureImporterModel(UploaderViewModel):
 
 class ConnectivityMeasureImporterForm(ABCUploaderForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(ConnectivityMeasureImporterForm, self).__init__(prefix, project_id)
+    def __init__(self):
+        super(ConnectivityMeasureImporterForm, self).__init__()
 
-        self.data_file = TraitUploadField(ConnectivityMeasureImporterModel.data_file, '.mat', self, name='data_file')
-        self.dataset_name = StrField(ConnectivityMeasureImporterModel.dataset_name, self, name='dataset_name')
-        self.connectivity = TraitDataTypeSelectField(ConnectivityMeasureImporterModel.connectivity, self,
-                                                     name='connectivity')
+        self.data_file = TraitUploadField(ConnectivityMeasureImporterModel.data_file, '.mat', 'data_file')
+        self.dataset_name = StrField(ConnectivityMeasureImporterModel.dataset_name, 'dataset_name')
+        self.connectivity = TraitDataTypeSelectField(ConnectivityMeasureImporterModel.connectivity, name='connectivity')
 
     @staticmethod
     def get_view_model():
@@ -116,17 +115,17 @@ class ConnectivityMeasureImporter(ABCUploader):
                                       ' contains %s nodes' % (node_count, connectivity.number_of_regions))
 
             measures = []
+            self.generic_attributes.user_tag_2 = "conn_%d" % node_count
+
             for i in range(measurement_count):
                 cm_data = data[i, :]
 
                 measure = ConnectivityMeasure()
                 measure.array_data = cm_data
                 measure.connectivity = connectivity
-                measure.title = "Measure for Conn with %d nods" % (node_count)
+                measure.title = "Measure %d for Connectivity with %d nodes." % ((i + 1), node_count)
 
                 cm_idx = h5.store_complete(measure, self.storage_path)
-                cm_idx.user_tag_2 = "nr.-%d" % (i + 1)
-                cm_idx.user_tag_3 = "conn_%d" % node_count
                 measures.append(cm_idx)
             return measures
 
