@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -33,17 +33,17 @@
 .. moduleauthor:: Mihai Andrei <mihai.andrei@codemart.ro>
 """
 import csv
+
 import numpy
+from tvb.adapters.datatypes.db.connectivity import ConnectivityIndex
 from tvb.basic.logger.builder import get_logger
+from tvb.core.adapters.abcuploader import ABCUploader, ABCUploaderForm
+from tvb.core.adapters.exceptions import LaunchException
+from tvb.core.neocom import h5
+from tvb.core.neotraits.forms import TraitUploadField, SelectField, TraitDataTypeSelectField
 from tvb.core.neotraits.uploader_view_model import UploaderViewModel
 from tvb.core.neotraits.view_model import Str, DataTypeGidAttr
 from tvb.datatypes.connectivity import Connectivity
-from tvb.core.adapters.exceptions import LaunchException
-from tvb.core.adapters.abcuploader import ABCUploader, ABCUploaderForm
-from tvb.core.entities.file.files_helper import FilesHelper
-from tvb.adapters.datatypes.db.connectivity import ConnectivityIndex
-from tvb.core.neotraits.forms import TraitUploadField, SelectField, TraitDataTypeSelectField
-from tvb.core.neocom import h5
 
 
 class CSVConnectivityParser(object):
@@ -142,16 +142,16 @@ class CSVConnectivityImporterModel(UploaderViewModel):
 
 class CSVConnectivityImporterForm(ABCUploaderForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(CSVConnectivityImporterForm, self).__init__(prefix, project_id)
+    def __init__(self):
+        super(CSVConnectivityImporterForm, self).__init__()
 
-        self.weights = TraitUploadField(CSVConnectivityImporterModel.weights, '.csv', self, name='weights')
-        self.weights_delimiter = SelectField(CSVConnectivityImporterModel.weights_delimiter, self,
-                                             name='weights_delimiter', choices=DELIMITER_OPTIONS)
-        self.tracts = TraitUploadField(CSVConnectivityImporterModel.tracts, ['.csv'], self, name='tracts')
-        self.tracts_delimiter = SelectField(CSVConnectivityImporterModel.tracts_delimiter, self,
-                                            name='tracts_delimiter', choices=DELIMITER_OPTIONS)
-        self.input_data = TraitDataTypeSelectField(CSVConnectivityImporterModel.input_data, self, name='input_data')
+        self.weights = TraitUploadField(CSVConnectivityImporterModel.weights, '.csv', 'weights')
+        self.weights_delimiter = SelectField(CSVConnectivityImporterModel.weights_delimiter, name='weights_delimiter',
+                                             choices=DELIMITER_OPTIONS)
+        self.tracts = TraitUploadField(CSVConnectivityImporterModel.tracts, ['.csv'], 'tracts')
+        self.tracts_delimiter = SelectField(CSVConnectivityImporterModel.tracts_delimiter, name='tracts_delimiter',
+                                            choices=DELIMITER_OPTIONS)
+        self.input_data = TraitDataTypeSelectField(CSVConnectivityImporterModel.input_data, 'input_data')
 
     @staticmethod
     def get_view_model():
@@ -205,13 +205,13 @@ class CSVConnectivityImporter(ABCUploader):
         """
         weights_matrix = self._read_csv_file(view_model.weights, view_model.weights_delimiter)
         tract_matrix = self._read_csv_file(view_model.tracts, view_model.tracts_delimiter)
-        FilesHelper.remove_files([view_model.weights, view_model.tracts])
-        input_data = self.load_entity_by_gid(view_model.input_data)
-        if weights_matrix.shape[0] != input_data.number_of_regions:
+        self.storage_interface.remove_files([view_model.weights, view_model.tracts])
+        conn_index = self.load_entity_by_gid(view_model.input_data)
+        if weights_matrix.shape[0] != conn_index.number_of_regions:
             raise LaunchException("The csv files define %s nodes but the connectivity you selected as reference "
-                                  "has only %s nodes." % (weights_matrix.shape[0], input_data.number_of_regions))
+                                  "has only %s nodes." % (weights_matrix.shape[0], conn_index.number_of_regions))
 
-        input_connectivity = h5.load_from_index(input_data)
+        input_connectivity = h5.load_from_index(conn_index)
 
         result = Connectivity()
         result.centres = input_connectivity.centres
