@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -89,13 +89,13 @@ class NIFTIImporterModel(UploaderViewModel):
 
 class NIFTIImporterForm(ABCUploaderForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(NIFTIImporterForm, self).__init__(prefix, project_id)
+    def __init__(self):
+        super(NIFTIImporterForm, self).__init__()
 
-        self.data_file = TraitUploadField(NIFTIImporterModel.data_file, ('.nii', '.gz', '.zip'), self, name='data_file')
-        self.apply_corrections = BoolField(NIFTIImporterModel.apply_corrections, self, name='apply_corrections')
-        self.mappings_file = TraitUploadField(NIFTIImporterModel.mappings_file, '.txt', self, name='mappings_file')
-        self.connectivity = TraitDataTypeSelectField(NIFTIImporterModel.connectivity, self, name='connectivity')
+        self.data_file = TraitUploadField(NIFTIImporterModel.data_file, ('.nii', '.gz', '.zip'), 'data_file')
+        self.apply_corrections = BoolField(NIFTIImporterModel.apply_corrections, name='apply_corrections')
+        self.mappings_file = TraitUploadField(NIFTIImporterModel.mappings_file, '.txt', 'mappings_file')
+        self.connectivity = TraitDataTypeSelectField(NIFTIImporterModel.connectivity, name='connectivity')
 
     @staticmethod
     def get_view_model():
@@ -214,7 +214,7 @@ class NIFTIImporter(ABCUploader):
 
             self.log.info("Imported RM with values in interval [%d - %d]" % (data.min(), data.max()))
             if not_matched:
-                self.log.warn("Not matched regions will be considered background: %s" % not_matched)
+                self.log.warning("Not matched regions will be considered background: %s" % not_matched)
                 if not apply_corrections:
                     raise ValidationException("Not matched regions were identified %s" % not_matched)
 
@@ -240,7 +240,7 @@ class NIFTIImporter(ABCUploader):
             title = "NIFTI Import - " + os.path.split(view_model.data_file)[1]
 
             if view_model.connectivity is not None:
-                connectivity_index = self.load_entity_by_gid(view_model.connectivity.hex)
+                connectivity_index = self.load_entity_by_gid(view_model.connectivity)
                 rm = self._create_region_map(volume_ht, connectivity_index, view_model.apply_corrections,
                                              view_model.mappings_file, view_model.title)
                 return [volume_idx, rm]
