@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -38,6 +38,7 @@ A Javascript displayer for time series, using SVG.
 import json
 from abc import ABCMeta
 from six import add_metaclass
+
 from tvb.adapters.datatypes.h5.time_series_h5 import TimeSeriesRegionH5, TimeSeriesSensorsH5, TimeSeriesH5
 from tvb.core.entities.filters.chain import FilterChain
 from tvb.core.adapters.abcadapter import ABCAdapterForm
@@ -59,10 +60,10 @@ class TimeSeriesModel(ViewModel):
 
 class TimeSeriesForm(ABCAdapterForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(TimeSeriesForm, self).__init__(prefix, project_id, False)
+    def __init__(self):
+        super(TimeSeriesForm, self).__init__()
 
-        self.time_series = TraitDataTypeSelectField(TimeSeriesModel.time_series, self, name='time_series',
+        self.time_series = TraitDataTypeSelectField(TimeSeriesModel.time_series, name='time_series',
                                                     conditions=self.get_filters())
 
     @staticmethod
@@ -109,8 +110,7 @@ class ABCSpaceDisplayer(ABCDisplayer):
         """
         if isinstance(ts_h5, TimeSeriesSensorsH5):
             sensors_gid = ts_h5.sensors.load()
-            sensors_idx = self.load_entity_by_gid(sensors_gid.hex)
-            with h5.h5_file_for_index(sensors_idx) as sensors_h5:
+            with h5.h5_file_for_gid(sensors_gid) as sensors_h5:
                 labels = sensors_h5.labels.load()
                 # TODO uncomment this when the UI component will be able to scale for many groups
                 # if isinstance(ts_h5, TimeSeriesSEEGH5):
@@ -133,16 +133,15 @@ class ABCSpaceDisplayer(ABCDisplayer):
             connectivity_gid = ts_h5.connectivity.load()
             if connectivity_gid is None:
                 return []
-            conn_idx = self.load_entity_by_gid(connectivity_gid.hex)
-            with h5.h5_file_for_index(conn_idx) as conn_h5:
+
+            with h5.h5_file_for_gid(connectivity_gid) as conn_h5:
                 return list(conn_h5.region_labels.load())
 
         if isinstance(ts_h5, TimeSeriesSensorsH5):
             sensors_gid = ts_h5.sensors.load()
             if sensors_gid is None:
                 return []
-            sensors_idx = self.load_entity_by_gid(sensors_gid.hex)
-            with h5.h5_file_for_index(sensors_idx) as sensors_h5:
+            with h5.h5_file_for_gid(sensors_gid) as sensors_h5:
                 return list(sensors_h5.labels.load())
 
         return ts_h5.get_space_labels()
@@ -163,11 +162,11 @@ class TimeSeriesDisplay(ABCSpaceDisplayer):
         return -1
 
     def _launch(self, view_model, figsize, preview=False):
-        time_series_index = self.load_entity_by_gid(view_model.time_series.hex)
+        time_series_index = self.load_entity_by_gid(view_model.time_series)
         h5_file = h5.h5_file_for_index(time_series_index)
         assert isinstance(h5_file, TimeSeriesH5)
         shape = list(h5_file.read_data_shape())
-        ts = h5_file.storage_manager.get_data('time')
+        ts = h5_file.time.load()
         state_variables = time_series_index.get_labels_for_dimension(1)
         labels = self.get_space_labels(h5_file)
 
@@ -197,7 +196,3 @@ class TimeSeriesDisplay(ABCSpaceDisplayer):
         # type: (TimeSeriesModel) -> dict
         """Construct data for visualization and launch it."""
         return self._launch(view_model, None)
-
-    def generate_preview(self, view_model, figure_size=None):
-        # type: (TimeSeriesModel, (int, int)) -> dict
-        return self._launch(view_model, figsize=figure_size, preview=True)
