@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -30,13 +30,14 @@
 
 import abc
 import json
+import typing
 import uuid
+
 import numpy
 import scipy.sparse
-import typing
 from tvb.basic.neotraits.api import HasTraits, Attr, NArray, Range
-from tvb.core.entities.file.exceptions import MissingDataSetException
 from tvb.datatypes import equations
+from tvb.storage.h5.file.exceptions import MissingDataSetException
 
 
 class Accessor(object, metaclass=abc.ABCMeta):
@@ -131,34 +132,43 @@ class DataSetMetaData(object):
     """
 
     # noinspection PyShadowingBuiltins
-    def __init__(self, min, max, mean):
+    def __init__(self, min, max, mean, is_finite=True, has_complex=False):
         self.min, self.max, self.mean = min, max, mean
+        self.is_finite = is_finite
+        self.has_complex = has_complex
 
     @classmethod
     def from_array(cls, array):
         try:
-            return cls(min=array.min(), max=array.max(), mean=array.mean())
+            return cls(min=array.min(), max=array.max(), mean=array.mean(),
+                       is_finite=numpy.isfinite(array).all().item(),
+                       has_complex=numpy.iscomplex(array).any().item())
         except (TypeError, ValueError):
             # likely a string array
             return cls(min=None, max=None, mean=None)
 
     @classmethod
     def from_dict(cls, dikt):
-        return cls(min=dikt['Minimum'], max=dikt['Maximum'], mean=dikt['Mean'])
+        return cls(min=dikt['Minimum'], max=dikt['Maximum'], mean=dikt['Mean'],
+                   is_finite=dikt['IsFinite'], has_complex=dikt["HasComplex"])
 
     def to_dict(self):
-        return {'Minimum': self.min, 'Maximum': self.max, 'Mean': self.mean}
+        return {'Minimum': self.min, 'Maximum': self.max, 'Mean': self.mean,
+                'IsFinite': self.is_finite, 'HasComplex': self.has_complex}
 
     def merge(self, other):
         self.min = min(self.min, other.min)
         self.max = max(self.max, other.max)
         self.mean = (self.mean + other.mean) / 2
+        self.is_finite = self.is_finite and other.is_finite
+        self.has_complex = self.has_complex or other.has_complex
 
 
 class DataSet(Accessor):
     """
     A dataset in a h5 file that corresponds to a traited NArray.
     """
+
     def __init__(self, trait_attribute, h5file, name=None, expand_dimension=-1):
         # type: (NArray, H5File, str, int) -> None
         """
@@ -172,28 +182,31 @@ class DataSet(Accessor):
         """
         super(DataSet, self).__init__(trait_attribute, h5file, name)
         self.expand_dimension = expand_dimension
+        # Cache metadata for expandable DataSets to avoid multiple reads/writes at append time
+        self.meta = None
 
     def append(self, data, close_file=True, grow_dimension=None):
         # type: (numpy.ndarray, bool, int) -> None
+        """
+        Method to be called when it is necessary to write slices of data for a large dataset, eg. TimeSeries.
+        Metdata for such datasets is written only at file close time, see H5File.close method.
+        """
         if not grow_dimension:
             grow_dimension = self.expand_dimension
         self.owner.storage_manager.append_data(
-            self.field_name,
             data,
+            self.field_name,
             grow_dimension=grow_dimension,
             close_file=close_file
         )
         # update the cached array min max metadata values
         new_meta = DataSetMetaData.from_array(numpy.array(data))
-        meta_dict = self.owner.storage_manager.get_metadata(self.field_name)
-        if meta_dict:
-            meta = DataSetMetaData.from_dict(meta_dict)
-            meta.merge(new_meta)
+        if self.meta:
+            self.meta.merge(new_meta)
         else:
             # this must be a new file, nothing to merge, set the new meta
-            meta = new_meta
-        self.owner.storage_manager.set_metadata(meta.to_dict(), self.field_name)
-
+            self.meta = new_meta
+            self.owner.expandable_datasets.append(self)
 
     def store(self, data):
         # type: (numpy.ndarray) -> None
@@ -202,7 +215,7 @@ class DataSet(Accessor):
         if data is None:
             return
 
-        self.owner.storage_manager.store_data(self.field_name, data)
+        self.owner.storage_manager.store_data(data, self.field_name)
         # cache some array information
         self.owner.storage_manager.set_metadata(
             DataSetMetaData.from_array(data).to_dict(),
@@ -217,7 +230,7 @@ class DataSet(Accessor):
 
     def __getitem__(self, data_slice):
         # type: (typing.Tuple[slice, ...]) -> numpy.ndarray
-        return self.owner.storage_manager.get_data(self.field_name, data_slice)
+        return self.owner.storage_manager.get_data(self.field_name, data_slice=data_slice)
 
     @property
     def shape(self):
@@ -230,6 +243,8 @@ class DataSet(Accessor):
         This cache is useful for large, expanding datasets,
         when we want to avoid loading the whole dataset just to compute a max.
         """
+        if self in self.owner.expandable_datasets:
+            return self.meta
         meta = self.owner.storage_manager.get_metadata(self.field_name)
         return DataSetMetaData.from_dict(meta)
 
@@ -282,6 +297,7 @@ class Reference(Uuid):
     A reference to another h5 file
     Corresponds to a contained datatype
     """
+
     def store(self, val):
         # type: (HasTraits) -> None
         """
@@ -349,7 +365,6 @@ class SparseMatrixMetaData(DataSetMetaData):
         }
 
 
-
 class SparseMatrix(Accessor):
     """
     Stores and loads a scipy.sparse csc or csr matrix in h5.
@@ -369,18 +384,18 @@ class SparseMatrix(Accessor):
             raise TypeError("expected scipy.sparse.spmatrix, got {}".format(type(mtx)))
 
         self.owner.storage_manager.store_data(
-            'data',
             mtx.data,
+            'data',
             where=self.field_name
         )
         self.owner.storage_manager.store_data(
-            'indptr',
             mtx.indptr,
+            'indptr',
             where=self.field_name
         )
         self.owner.storage_manager.store_data(
-            'indices',
             mtx.indices,
+            'indices',
             where=self.field_name
         )
         self.owner.storage_manager.set_metadata(
@@ -414,7 +429,6 @@ class SparseMatrix(Accessor):
         return mtx
 
 
-
 class Json(Scalar):
     """
     A python json like data structure accessor
@@ -439,6 +453,7 @@ class Json(Scalar):
             return self.json_decoder().decode(val)
         return json.loads(val)
 
+
 class JsonRange(Scalar):
     """
     Stores and loads a Range in the form of a json in h5.
@@ -484,4 +499,5 @@ class JsonFinal(Json):
             return dict_array
 
     def __init__(self, trait_attribute, h5file, name=None):
-        super(JsonFinal, self).__init__(trait_attribute, h5file, name, self.StateVariablesEncoder, self.StateVariablesDecoder)
+        super(JsonFinal, self).__init__(trait_attribute, h5file, name, self.StateVariablesEncoder,
+                                        self.StateVariablesDecoder)
