@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -27,27 +27,26 @@
 #   Frontiers in Neuroinformatics (7:10. doi: 10.3389/fninf.2013.00010)
 #
 #
-
-import importlib
-import typing
-import os.path
-import uuid
 from datetime import datetime
+import importlib
+import os.path
+import typing
+import uuid
 import numpy
 import scipy.sparse
+
 from tvb.basic.logger.builder import get_logger
-from tvb.basic.neotraits._attr import Final
-from tvb.basic.neotraits.ex import TraitFinalAttributeError
-from tvb.core.entities.file.exceptions import MissingDataSetException
-from tvb.core.entities.file.hdf5_storage_manager import HDF5StorageManager
+from tvb.basic.neotraits.api import Final
 from tvb.basic.neotraits.api import HasTraits, Attr, List, NArray, Range
+from tvb.basic.neotraits.ex import TraitFinalAttributeError
 from tvb.core.entities.generic_attributes import GenericAttributes
-from tvb.core.neotraits._h5accessors import Uuid, Scalar, Accessor, DataSet, Reference, JsonFinal, Json, JsonRange, \
-    EquationScalar, \
-    SparseMatrix, ReferenceList
+from tvb.core.neotraits.h5 import EquationScalar, SparseMatrix, ReferenceList
+from tvb.core.neotraits.h5 import Uuid, Scalar, Accessor, DataSet, Reference, JsonFinal, Json, JsonRange
 from tvb.core.neotraits.view_model import DataTypeGidAttr
-from tvb.core.utils import date2string, string2date
+from tvb.core.utils import string2date, date2string
 from tvb.datatypes.equations import Equation
+from tvb.storage.h5.file.exceptions import MissingDataSetException
+from tvb.storage.storage_interface import StorageInterface
 
 LOGGER = get_logger(__name__)
 
@@ -64,14 +63,14 @@ class H5File(object):
     def __init__(self, path):
         # type: (str) -> None
         self.path = path
-        storage_path, file_name = os.path.split(path)
-        self.storage_manager = HDF5StorageManager(storage_path, file_name)
+        self.storage_manager = StorageInterface.get_storage_manager(self.path)
         # would be nice to have an opened state for the chunked api instead of the close_file=False
 
         # common scalar headers
         self.gid = Uuid(HasTraits.gid, self)
         self.written_by = Scalar(Attr(str), self, name=self.KEY_WRITTEN_BY)
         self.create_date = Scalar(Attr(str), self, name='create_date')
+        self.type = Scalar(Attr(str), self, name='type')
 
         # Generic attributes descriptors
         self.generic_attributes = GenericAttributes()
@@ -79,17 +78,20 @@ class H5File(object):
         self.is_nan = Scalar(Attr(bool), self, name='is_nan')
         self.subject = Scalar(Attr(str), self, name='subject')
         self.state = Scalar(Attr(str), self, name='state')
-        self.type = Scalar(Attr(str), self, name='type')
         self.user_tag_1 = Scalar(Attr(str), self, name='user_tag_1')
         self.user_tag_2 = Scalar(Attr(str), self, name='user_tag_2')
         self.user_tag_3 = Scalar(Attr(str), self, name='user_tag_3')
         self.user_tag_4 = Scalar(Attr(str), self, name='user_tag_4')
         self.user_tag_5 = Scalar(Attr(str), self, name='user_tag_5')
+        self.operation_tag = Scalar(Attr(str, required=False), self, name='operation_tag')
+        self.parent_burst = Uuid(Attr(uuid.UUID, required=False), self, name='parent_burst')
         self.visible = Scalar(Attr(bool), self, name='visible')
         self.metadata_cache = None
+        # Keep a list with datasets for which we should write metadata before closing the file
+        self.expandable_datasets = []
 
-        if not self.storage_manager.is_valid_hdf5_file():
-            self.written_by.store(self.__class__.__module__ + '.' + self.__class__.__name__)
+        if not self.storage_manager.is_valid_tvb_file():
+            self.written_by.store(self.get_class_path())
             self.is_new_file = True
 
     @classmethod
@@ -99,6 +101,9 @@ class H5File(object):
     def read_subtype_attr(self):
         return None
 
+    def get_class_path(self):
+        return self.__class__.__module__ + '.' + self.__class__.__name__
+
     def iter_accessors(self):
         # type: () -> typing.Generator[Accessor]
         for accessor in self.__dict__.values():
@@ -117,6 +122,8 @@ class H5File(object):
         self.close()
 
     def close(self):
+        for dataset in self.expandable_datasets:
+            self.storage_manager.set_metadata(dataset.meta.to_dict(), dataset.field_name)
         self.storage_manager.close_file()
 
     def store(self, datatype, scalars_only=False, store_references=True):
@@ -179,13 +186,15 @@ class H5File(object):
         self.is_nan.store(self.generic_attributes.is_nan)
         self.subject.store(self.generic_attributes.subject)
         self.state.store(self.generic_attributes.state)
-        self.type.store(self.generic_attributes.type)
         self.user_tag_1.store(self.generic_attributes.user_tag_1)
         self.user_tag_2.store(self.generic_attributes.user_tag_2)
         self.user_tag_3.store(self.generic_attributes.user_tag_3)
         self.user_tag_4.store(self.generic_attributes.user_tag_4)
         self.user_tag_5.store(self.generic_attributes.user_tag_5)
+        self.operation_tag.store(self.generic_attributes.operation_tag)
         self.visible.store(self.generic_attributes.visible)
+        if self.generic_attributes.parent_burst is not None:
+            self.parent_burst.store(uuid.UUID(self.generic_attributes.parent_burst))
 
     def load_generic_attributes(self):
         # type: () -> GenericAttributes
@@ -193,7 +202,6 @@ class H5File(object):
         self.generic_attributes.is_nan = self.is_nan.load()
         self.generic_attributes.subject = self.subject.load()
         self.generic_attributes.state = self.state.load()
-        self.generic_attributes.type = self.type.load()
         self.generic_attributes.user_tag_1 = self.user_tag_1.load()
         self.generic_attributes.user_tag_2 = self.user_tag_2.load()
         self.generic_attributes.user_tag_3 = self.user_tag_3.load()
@@ -201,17 +209,32 @@ class H5File(object):
         self.generic_attributes.user_tag_5 = self.user_tag_5.load()
         self.generic_attributes.visible = self.visible.load()
         self.generic_attributes.create_date = string2date(str(self.create_date.load())) or None
+        try:
+            self.generic_attributes.operation_tag = self.operation_tag.load()
+        except MissingDataSetException:
+            self.generic_attributes.operation_tag = None
+        try:
+            burst = self.parent_burst.load()
+            self.generic_attributes.parent_burst = burst.hex if burst is not None else None
+        except MissingDataSetException:
+            self.generic_attributes.parent_burst = None
         return self.generic_attributes
 
-    def gather_references(self):
+    def gather_references(self, datatype_cls=None):
         ret = []
         for accessor in self.iter_accessors():
+            trait_attribute = None
+            if datatype_cls:
+                if hasattr(datatype_cls, accessor.field_name):
+                    trait_attribute = getattr(datatype_cls, accessor.field_name)
+            if not trait_attribute:
+                trait_attribute = accessor.trait_attribute
             if isinstance(accessor, Reference):
-                ret.append((accessor.trait_attribute, accessor.load()))
+                ret.append((trait_attribute, accessor.load()))
             if isinstance(accessor, ReferenceList):
                 hex_gids = accessor.load()
                 gids = [uuid.UUID(hex_gid) for hex_gid in hex_gids]
-                ret.append((accessor.trait_attribute, gids))
+                ret.append((trait_attribute, gids))
         return ret
 
     def determine_datatype_from_file(self):
@@ -224,10 +247,7 @@ class H5File(object):
     @staticmethod
     def determine_type(path):
         # type: (str) -> typing.Type[HasTraits]
-        base_dir, fname = os.path.split(path)
-        storage_manager = HDF5StorageManager(base_dir, fname)
-        meta = storage_manager.get_metadata()
-        type_class_fqn = meta.get('type')
+        type_class_fqn = H5File.get_metadata_param(path, 'type')
         if type_class_fqn is None:
             return HasTraits
         package, cls_name = type_class_fqn.rsplit('.', 1)
@@ -235,13 +255,24 @@ class H5File(object):
         cls = getattr(module, cls_name)
         return cls
 
+    @staticmethod
+    def get_metadata_param(path, param):
+        meta = StorageInterface.get_storage_manager(path).get_metadata()
+        return meta.get(param)
+
+    def store_metadata_param(self, key, value):
+        self.storage_manager.set_metadata({key: value})
+
+    @staticmethod
+    def remove_metadata_param(file_path, param):
+        storage_manager = StorageInterface.get_storage_manager(file_path)
+        if param in storage_manager.get_metadata():
+            storage_manager.remove_metadata(param)
+
     @staticmethod
     def h5_class_from_file(path):
         # type: (str) -> typing.Type[H5File]
-        base_dir, fname = os.path.split(path)
-        storage_manager = HDF5StorageManager(base_dir, fname)
-        meta = storage_manager.get_metadata()
-        h5file_class_fqn = meta.get(H5File.KEY_WRITTEN_BY)
+        h5file_class_fqn = H5File.get_metadata_param(path, H5File.KEY_WRITTEN_BY)
         if h5file_class_fqn is None:
             return H5File(path)
         package, cls_name = h5file_class_fqn.rsplit('.', 1)
@@ -261,6 +292,7 @@ class H5File(object):
 
 class ViewModelH5(H5File):
 
+    # TODO  it will be good to be able to just call with H5File.from_file(h5_path) as f for ViewModelH5 also
     def __init__(self, path, view_model):
         super(ViewModelH5, self).__init__(path)
         self.view_model = type(view_model)
@@ -303,17 +335,17 @@ class ViewModelH5(H5File):
                 ref = Accessor(attr, self)
             setattr(self, attr.field_name, ref)
 
-    def gather_references_by_uuid(self):
+    def gather_datatypes_references(self):
         """
         Mind that ViewModelH5 stores references towards ViewModel objects (eg. Coupling) as Reference attributes, and
         references towards existent Datatypes (eg. Connectivity) as Uuid.
         Thus, the method gather_references will return only references towards other ViewModels, and we need this
-        method to gather also the other references.
+        method to gather also datatypes references.
         """
         ret = []
         for accessor in self.iter_accessors():
             if isinstance(accessor, Uuid) and not isinstance(accessor, Reference):
-                if accessor.field_name is 'gid':
+                if accessor.field_name in ('gid', 'parent_burst', 'operation_group_gid'):
                     continue
                 ret.append((accessor.trait_attribute, accessor.load()))
-        return ret
\ No newline at end of file
+        return ret
